/* eslint-disable react/no-array-index-key */
import React, { useRef, useEffect, useState } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames/bind';
import throttle from 'lodash.throttle';
import gaDataset from '../../../utils/gaDataset';
import AnueLink from '../../../utils/anueLink';
import ArrowLeft from '../../../assets/icon-arrow-left.svg';
import ArrowRight from '../../../assets/icon-arrow-right.svg';
import { DIRECTION } from '../constants';
import styles from '../AnueNavigator.scss';

const cx = classNames.bind(styles);

const AnueSubMenu = ({ data, parentName, headerRef, parentRef, apiEnv, dataPrefix, expandSearchBar }) => {
  const ref = useRef();
  const menusRef = useRef();
  const offsetRef = useRef();

  const [needShowArrow, setNeedShowArrow] = useState({ left: false, right: false });

  const anueUrl = new AnueLink(apiEnv);

  /**
   * 對應設計稿，次選單的呈現主要有三種狀況
   * 1. 如果滑出的選單置中情況下，會超出 header 的邊界 (1200, 1024)，則直接對齊邊界
   * 2. 如果滑出的選單置中情況下，不會超出 header 的邊界，則置中對齊
   * 3. 如果選單本身的寬度就已經超過 header (1200, 1024)時，則超出邊界全版顯示
   * 設計稿: https://zpl.io/aMy3zN3
   */
  const handlePosition = (needRestore = true) => {
    if (!ref.current) return;

    // 當樣式設定 `display: none` 時，該物件本身的寬高都是為 0，是不會被繪製出來的
    // 所以先將物件顯示在畫面上，讓 Browser 可以繪製出選單的寬度和位置
    // 之後再將物件隱藏
    // ref.current.style.display = 'flex';

    if (needRestore) {
      ref.current.style.width = 'auto';
      setNeedShowArrow(prevState => ({ ...prevState, left: false, right: false }));
    }

    const paddingValue = 16;
    // 上層 Header，用來判斷內容區域的邊界值和寬度
    const headerBoundary = headerRef.current.getBoundingClientRect();
    // 上層 Channel 物件，用來計算次選單顯示座標定位
    const parentBoundary = parentRef.current.getBoundingClientRect();
    // 次選單物件，用來取得目前選單寬度和預設顯示座標
    const subMenuBoundary = ref.current.getBoundingClientRect();

    // 次選單寬度超出 header 的邊界，全版顯示
    if (subMenuBoundary.width >= headerBoundary.width) {
      const isOverScreenWidth = subMenuBoundary.width - document.documentElement.offsetWidth >= 0;

      // 顯示滿版情況下的左右箭頭
      if (isOverScreenWidth) {
        setNeedShowArrow(prevState => ({ ...prevState, right: true }));

        // 次選單需要滿版顯示
        offsetRef.current = {
          left: `-${parentBoundary.x}px`,
          width: `${document.documentElement.offsetWidth}px`,
        };
      } else {
        // 當次選單寬度超出 header 寬度時，但沒有超出螢幕寬度時，採置中對齊 - [QA-1647]
        const subMenuCenterPosition =
          Math.abs(parentBoundary.x - headerBoundary.x) +
          Math.round((subMenuBoundary.width - headerBoundary.width) / 2);

        offsetRef.current = { left: `-${subMenuCenterPosition}px` };
      }
    } else {
      const subMenuAndHeaderPositionDiff = Math.abs(parentBoundary.x - headerBoundary.x);

      if (subMenuAndHeaderPositionDiff > 0) {
        const subMenuPosition = Math.round(subMenuBoundary.width / 2) - paddingValue * 2;

        offsetRef.current =
          subMenuAndHeaderPositionDiff < subMenuPosition
            ? { left: `-${subMenuAndHeaderPositionDiff}px` }
            : { left: `-${subMenuPosition}px` };
      }
    }

    // ref.current.style.display = '';
  };

  /**
   * 畫面 Resize 需重新計算次選單的座標位置
   */
  useEffect(() => {
    window.addEventListener('resize', throttle(handlePosition, 200));

    return () => window.removeEventListener('resize', handlePosition);
  }, []);

  /**
   * 配合主導覽列上的搜尋框展開，需要重新計算次選單的座標位置
   */
  useEffect(
    () => {
      handlePosition(false);
    },
    [expandSearchBar]
  );

  /**
   * 次選單項目超出畫面時的左右移動處理
   */
  const moveSubMenuHandler = direction => {
    let currentDistance = menusRef.current.scrollLeft;

    const needMoveCount = Math.ceil(menusRef.current.scrollWidth / menusRef.current.offsetWidth) - 1;
    const moveDistance = (menusRef.current.scrollWidth - menusRef.current.offsetWidth) / needMoveCount;

    currentDistance = direction === DIRECTION.RIGHT ? currentDistance + moveDistance : currentDistance - moveDistance;
    menusRef.current.scrollLeft = currentDistance;

    if (currentDistance > 0) {
      setNeedShowArrow(prevState => ({ ...prevState, left: true }));
    } else if (currentDistance <= 0) {
      setNeedShowArrow(prevState => ({ ...prevState, left: false }));
    }

    const currentScrollWidth = menusRef.current.offsetWidth + currentDistance;

    if (currentScrollWidth < menusRef.current.scrollWidth) {
      setNeedShowArrow(prevState => ({ ...prevState, right: true }));
    } else if (currentScrollWidth >= menusRef.current.scrollWidth) {
      setNeedShowArrow(prevState => ({ ...prevState, right: false }));
    }
  };

  /**
   * React events are actually Synthetic Events, not Native Events.
   * Event delegation: React doesn't actually attach event handlers to the nodes themselves.
   * When React starts up, it starts listening for all events at the top level using a single event listener.
   * When a component is mounted or unmounted, the event handlers are simply added or removed from an internal mapping.
   * When an event occurs, React knows how to dispatch it using this mapping.
   * When there are no event handlers left in the mapping,
   * React's event handlers are simple no-ops.
   *
   * Use `Event.stopImmediatePropagation`
   *
   * https://stackoverflow.com/questions/36316846/react-onclick-and-preventdefault-link-refresh-redirect
   */
  const onMoveSubMenuLeft = e => {
    e.stopPropagation();
    e.nativeEvent.stopImmediatePropagation();

    moveSubMenuHandler(DIRECTION.LEFT);
  };

  const onMoveSubMenuRight = e => {
    e.stopPropagation();
    e.nativeEvent.stopImmediatePropagation();

    moveSubMenuHandler(DIRECTION.RIGHT);
  };

  /**
   * 展開第二層選單項目
   * 淺灰色連結
   */
  const renderSubSecondMenuItems = (parent, first, second) => {
    // parentName 是配合靜態頁面生成使用，靜態生成時無法動態取得 parentRef 的值，因為是在 Server 端生成，不是在 Browser 上建立
    const rootTitle = parentName || (parentRef && parentRef.current && parentRef.current.children[0].innerText);

    const { subItems: items } = parent;
    const secondMenus = items.map((item, index) => (
      <li key={`sub_main_menu_${first}_${second}_${index}`}>
        <a
          className={cx({ highlight: item.highlight })}
          href={anueUrl.generateLink(item.url)}
          target={item.blank ? '_blank' : '_self'}
          rel="noopener noreferrer"
          {...gaDataset({
            dataPrefix,
            category: 'Nav',
            action: 'click',
            label: `${rootTitle}_${parent.title}_${item.title}`,
          })}
        >
          {item.title}
        </a>
      </li>
    ));

    return <ul className={cx('channel-item-sub-menu-second')}>{secondMenus}</ul>;
  };

  /**
   * 顯示每一欄的第一層選單項目
   * 粗體連結
   */
  const renderSubMainMenuItems = (items, first) => {
    if (items.length <= 0) return [];

    // parentName 是配合靜態頁面生成使用，靜態生成時無法動態取得 parentRef 的值，因為是在 Server 端生成，不是在 Browser 上建立
    const rootTitle = parentName || (parentRef && parentRef.current && parentRef.current.children[0].innerText);

    return items.map((item, index) => {
      const haveSecondMenu = item.subItems && item.subItems.length > 0;

      return (
        <li key={`sub_main_menu_${first}_${index}`}>
          {item.url !== '' ? (
            <a
              className={cx({ highlight: item.highlight })}
              href={anueUrl.generateLink(item.url)}
              target={item.blank ? '_blank' : '_self'}
              rel="noopener noreferrer"
              {...gaDataset({ dataPrefix, category: 'Nav', action: 'click', label: `${rootTitle}_${item.title}` })}
            >
              {item.title}
            </a>
          ) : (
            item.title
          )}
          {haveSecondMenu && renderSubSecondMenuItems(item, first, index)}
        </li>
      );
    });
  };

  /**
   * 依照第一層的 UL 個數，建立次選單中的欄
   */
  const renderSubMainMenuColumn = () => {
    if (data.length <= 0) return [];

    return data.map((menu, index) => {
      return <ul key={`sub_main_menu_${index}`}>{renderSubMainMenuItems(menu, index)}</ul>;
    });
  };

  return (
    <div ref={ref} className={cx('channel-item-sub-menu')} style={offsetRef.current}>
      {needShowArrow.left && (
        <div className={cx('channel-item-sub-menu-arrow', 'left')}>
          <img src={ArrowLeft} alt="Previous" onClick={onMoveSubMenuLeft} />
        </div>
      )}
      <div ref={menusRef} className={cx('channel-item-sub-menus')}>
        {renderSubMainMenuColumn()}
      </div>
      {needShowArrow.right && (
        <div className={cx('channel-item-sub-menu-arrow', 'right')}>
          <img src={ArrowRight} alt="Next" onClick={onMoveSubMenuRight} />
        </div>
      )}
    </div>
  );
};

const MenuItemProps = PropTypes.shape({
  title: PropTypes.string.isRequired,
  url: PropTypes.string.isRequired,
});

AnueSubMenu.propTypes = {
  data: PropTypes.arrayOf(MenuItemProps).isRequired,
  parentName: PropTypes.string,
  headerRef: PropTypes.element.isRequired,
  parentRef: PropTypes.element.isRequired,
  apiEnv: PropTypes.string.isRequired,
  dataPrefix: PropTypes.arrayOf(PropTypes.string),
  expandSearchBar: PropTypes.bool,
};

AnueSubMenu.defaultProps = {
  parentName: '',
  dataPrefix: ['data-global-ga'],
  expandSearchBar: false,
};

export default AnueSubMenu;
