/* eslint-disable space-infix-ops */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames/bind';
import AnueSearchInput from '../AnueSearchInput/AnueSearchInput';
import AnueSearchRealtimeResult from '../AnueSearchRealtimeResult/AnueSearchRealtimeResult';
import AnueApiFetch from '../../utils/AnueApiFetch';
import SearchResultHelper from './helpers';
import debounce from '../../utils/debounce';
import styles from './AnueSearch.scss';
import AnueLink from '../../utils/anueLink';
import AnueCookie from '../../utils/anueCookie';
import gaDataset from '../../utils/gaDataset';

const cx = classnames.bind(styles);

export default class AnueSearch extends Component {
  static propTypes = {
    maxRecentHistory: PropTypes.number,
    theme: PropTypes.string, // mobile | desktop
    // AnueSearchInputProps
    customAttribute: PropTypes.objectOf(PropTypes.any),
    customWrapperStyles: PropTypes.objectOf(PropTypes.any),
    customInputWrapperStyles: PropTypes.objectOf(PropTypes.any),
    customResultWrapperStyles: PropTypes.objectOf(PropTypes.any),
    customInputStyles: PropTypes.objectOf(PropTypes.any),
    customIconStyles: PropTypes.objectOf(PropTypes.any),
    shouldAlwaysDisplayInput: PropTypes.bool,
    shouldDisplayBackIcon: PropTypes.bool,
    onIconClick: PropTypes.func,
    onBackIconClick: PropTypes.func,
    onInputChange: PropTypes.func,
    onInputFocus: PropTypes.func,
    onInputBlur: PropTypes.func,
    onInputPressEnter: PropTypes.func,
    onSearchVisibilityChange: PropTypes.func,
    placeholder: PropTypes.string,
    apiEnv: PropTypes.string,
    aink: PropTypes.string,
    defaultValue: PropTypes.string,
    enableNewStyle: PropTypes.bool,
    dataPrefix: PropTypes.arrayOf(PropTypes.string),
  };
  static defaultProps = {
    maxRecentHistory: 10,
    theme: null,
    customAttribute: null,
    customWrapperStyles: null,
    customInputWrapperStyles: null,
    customResultWrapperStyles: null,
    customInputStyles: null,
    customIconStyles: null,
    customResultClassName: null,
    shouldAlwaysDisplayInput: false,
    shouldDisplayBackIcon: false,
    placeholder: '搜尋新聞、代碼或名稱',
    onIconClick: () => {},
    onBackIconClick: () => {},
    onInputChange: () => {},
    onInputFocus: () => {},
    onInputBlur: () => {},
    onInputPressEnter: () => {},
    onSearchVisibilityChange: () => {},
    apiEnv: 'prod',
    aink: null,
    defaultValue: null,
    enableNewStyle: false,
    dataPrefix: ['data-global-ga'],
  };
  constructor(props) {
    super(props);
    let defaultSearchString = props.defaultValue || '';

    if (typeof window !== 'undefined' && props.defaultValue === null) {
      const keywordQueryStringRegex = location.search && decodeURIComponent(location.search).match(/keyword=([^&]*)/);

      if (keywordQueryStringRegex && keywordQueryStringRegex[1]) {
        defaultSearchString = keywordQueryStringRegex[1];
      }
    }

    this.state = {
      isInputHidden: false,
      isInputFocus: false,
      hotTags: null,
      hotTrades: null,
      recentSearch: null,
      hotDrivers: null,
      searchString: defaultSearchString,
      tradeResult: null,
      newsResult: null,
      driversResult: null,
      hover: false,
    };

    this.request = new AnueApiFetch();
    this.anueUrlSingleton = new AnueLink(props.apiEnv);
    this.anueCookieSingleton = new AnueCookie(props.apiEnv);
    this.dataHelpers = SearchResultHelper(this.anueUrlSingleton);
  }

  componentDidMount() {
    if (typeof window !== 'undefined') {
      // The clicked event of AnueSearch is set to stopPropagation.
      // So click the anywhere outside of AnueSearch will hide the AnueRealtimeResult.
      window.addEventListener('click', this.handleHideSearchResult);
    }
  }

  componentDidUpdate(prevPrev, prevState) {
    const { theme } = this.props;
    const { hover } = this.state;

    if (typeof window !== 'undefined' && prevState.hover !== hover) {
      const bodyStyles = document.body.style;
      const screenWidth = window.innerWidth || document.documentElement.clientWidth;
      const isSmallMode = theme === 'mobile' || screenWidth < styles.mobileScreenWidth;

      if (hover && isSmallMode) {
        document.body.style.overflowY = 'hidden';
      } else if (bodyStyles) {
        document.body.style.overflowY = 'auto';
      }
    }
  }

  componentWillUnmount() {
    window.removeEventListener('click', this.handleHideSearchResult);
  }

  getSearchHistory = () => {
    const history = this.anueCookieSingleton.getSearchCookies();

    this.setState({
      recentSearch: this.dataHelpers.recentSearchDataHelper(history),
    });
  };

  storeSearchHistory = (string, shouldDelete) => {
    const { maxRecentHistory, apiEnv } = this.props;

    this.anueCookieSingleton.setSearchCookies({ string, shouldDelete, cookieLimit: maxRecentHistory, env: apiEnv });
    this.getSearchHistory();
  };

  removeAllTags = () => {
    const { apiEnv } = this.props;

    this.anueCookieSingleton.removeSearchCookies({ env: apiEnv });
    this.getSearchHistory();
  };

  handleClickRecentTag = tagName => {
    const { aink } = this.props;

    const link = this.anueUrlSingleton.generateSearchResultLink({
      keyword: tagName,
      backLink: aink,
    });

    location.href = link;
  };

  handleClickHotTag = tag => {
    if (tag && tag.tagName) {
      const keyword = tag.tagName;

      this.inputRef.value = keyword;
      this.inputRef.focus();
      this.handleSearchChange(keyword);
    }
  };

  handleClickRemoveTag = tagName => {
    this.storeSearchHistory(tagName, true);
  };

  handleFetch = debounce(async () => {
    const { searchString } = this.state;

    const rawResult = await this.request.get(
      `${this.anueUrlSingleton.anueSearchAllApiUrl}${encodeURIComponent(searchString)}`
    );
    const result = this.dataHelpers.searchTradesResponseHelper(rawResult);

    if (result && (result.quoteFunds || result.news || result.oldDrivers)) {
      this.setState({
        tradeResult: result.quoteFunds,
        newsResult: result.news,
        driversResult: result.oldDrivers,
      });
    }
  }, 250);

  handleSearchChange = value => {
    const { onInputChange, dataPrefix } = this.props;

    this.setState({ searchString: value });

    onInputChange(value);

    this.handleFetch();
    gaDataset({ dataPrefix, category: 'Searchall', action: 'click', label: value });
  };

  handlePressEnter = () => {
    const { onInputPressEnter, aink } = this.props;
    const { searchString } = this.state;

    onInputPressEnter();
    if (!!searchString && typeof window !== 'undefined') {
      this.storeSearchHistory(searchString);
      const link = this.anueUrlSingleton.generateSearchResultLink({ keyword: searchString, backLink: aink });

      location.href = link;
    }
  };

  handleClickSearchIcon = () => {
    const { onIconClick, aink } = this.props;
    const { searchString, hover } = this.state;

    onIconClick();
    if (!!searchString && typeof window !== 'undefined') {
      this.storeSearchHistory(searchString);
      const link = this.anueUrlSingleton.generateSearchResultLink({ keyword: searchString, backLink: aink });

      location.href = link;
    }
    if (hover) {
      this.setState({
        hover: false,
      });
    }
  };

  handleClickBackIcon = () => {
    const { onBackIconClick } = this.props;

    this.setState({
      hover: false,
    });

    onBackIconClick();
  };

  handleClickResultRow = () => {
    const { searchString } = this.state;

    if (!!searchString && typeof window !== 'undefined') {
      this.storeSearchHistory(searchString);
    }
  };

  handleSearchOnFocus = async () => {
    const { onInputFocus } = this.props;
    const { hotTags, hotTrades, hotDrivers } = this.state;

    this.setState({ hover: true });
    onInputFocus();

    if (!hotTags && !hotTrades && !hotDrivers) {
      const _hotTrades = await this.request.get(this.anueUrlSingleton.hotTradesApiUrl);
      const _hotTags = await this.request.get(this.anueUrlSingleton.hotTagsApiUrl);
      const _hotDrivers = await this.request.get(this.anueUrlSingleton.hotDriversApiUrl);

      this.getSearchHistory();
      this.setState({
        hotTags: this.dataHelpers.hotTagsResponseHelper(_hotTags),
        hotTrades: this.dataHelpers.hotTradesResponseHelper(_hotTrades),
        hotDrivers: this.dataHelpers.hotDriversResponseHelper(_hotDrivers),
      });
    }
  };

  handleHideSearchResult = () => {
    const { hover } = this.state;

    if (hover) {
      this.setState({
        hover: false,
      });
    }
  };

  handleSearchOnBlur = () => {
    const { onInputBlur } = this.props;

    onInputBlur();
  };

  handleStopWrapperClickPropagation = e => {
    e.stopPropagation();
  };

  render() {
    const {
      theme,
      shouldAlwaysDisplayInput,
      shouldDisplayBackIcon,
      placeholder,
      customAttribute,
      customWrapperStyles,
      customInputWrapperStyles,
      customResultWrapperStyles,
      customInputStyles,
      customIconStyles,
      onSearchVisibilityChange,
      aink,
      enableNewStyle,
      apiEnv,
    } = this.props;
    const {
      hotTags,
      hotTrades,
      hotDrivers,
      recentSearch,
      searchString,
      hover,
      tradeResult,
      newsResult,
      driversResult,
    } = this.state;
    const searchResultLink = this.anueUrlSingleton.generateSearchResultLink({ keyword: searchString, backLink: aink });

    let searchClassName = 'anue-search';
    let customInputClassName = null;

    if (enableNewStyle) {
      searchClassName = 'anue-new-search';
      customInputClassName = 'anue-new-search-input--wrapper';
    }

    return (
      <div
        className={cx(searchClassName, theme)}
        style={customWrapperStyles}
        onClick={this.handleStopWrapperClickPropagation}
      >
        <div className={cx('input-wrapper', { active: hover })}>
          <AnueSearchInput
            theme={theme}
            placeholder={placeholder}
            onInputChange={this.handleSearchChange}
            onInputFocus={this.handleSearchOnFocus}
            onInputBlur={this.handleSearchOnBlur}
            onInputPressEnter={this.handlePressEnter}
            onIconClick={this.handleClickSearchIcon}
            onBackIconClick={this.handleClickBackIcon}
            shouldAlwaysDisplayInput={shouldAlwaysDisplayInput || !!searchString}
            shouldDisplayBackIcon={shouldDisplayBackIcon}
            defaultValue={searchString}
            customWrapperStyles={customInputWrapperStyles}
            customInputStyles={customInputStyles}
            customIconStyles={customIconStyles}
            onDisplayChange={onSearchVisibilityChange}
            enableNewStyle={enableNewStyle}
            customInputClassName={customInputClassName}
            customAttribute={{
              ...customAttribute,
              ref: input => {
                this.inputRef = input;
              },
            }}
          />
        </div>
        <div className={cx('search-result', theme, { display: hover })}>
          <AnueSearchRealtimeResult
            hotTags={hotTags}
            hotTrades={hotTrades}
            hotDrivers={hotDrivers}
            recentSearch={recentSearch}
            searchString={searchString}
            tradeResult={tradeResult}
            newsResult={newsResult}
            driversResult={driversResult}
            onHotTagClick={this.handleClickHotTag}
            onRecentSearchRowClick={this.handleClickRecentTag}
            onRecentSearchRowRemoveClick={this.handleClickRemoveTag}
            onRecentSearchRemoveAllClick={this.removeAllTags}
            onResultSearchRowClick={this.handleClickResultRow}
            customWrapperStyles={customResultWrapperStyles}
            searchResultPageLink={searchResultLink}
            enableNewStyle={enableNewStyle}
            env={apiEnv}
          />
        </div>
      </div>
    );
  }
}
