import React from 'react';
import { mount } from 'enzyme';
import Compare from '../Compare';

describe('<Compare />', () => {
  let makeSubject;
  const defaultProps = {
    items: [
      { id: 'May the force be with you', name: 'May the force be with you - Jedi' },
      { id: 'You complete me', name: 'You complete me - Jerry Maguire' },
      { id: 'Let it go', name: 'Let it go - Lisa' },
      {
        id: 'With great power comes great responsibility',
        name: 'With great power comes great responsibility - Uncle Ben',
      },
    ],
    ctaCb: jest.fn(),
    clickItemCloseCb: jest.fn(),
    prerequisiteMsg: 'IP Man',
    regulationMsg: 'Slam Dunk',
    btnText: 'FaDaCai',
    isMobile: false,
  };

  beforeEach(() => {
    makeSubject = params => mount(<Compare {...defaultProps} {...params} />);
  });

  it('should render the compare component out of the viewpoint if no item', () => {
    let subject;

    subject = makeSubject();
    expect(subject.find('.comparison.display').length).toBe(1);

    subject = makeSubject({
      items: [],
    });
    expect(subject.find('.comparison.display').length).toBe(0);
  });

  it('should append the contents to the blocks based on the number of the prop item', () => {
    let subject;

    subject = makeSubject();
    expect(subject.find('.block').length).toBe(4);

    subject = makeSubject({ items: [] });
    expect(subject.find('.content').length).toBe(0);

    defaultProps.items.forEach((item, i) => {
      subject.setProps({
        items: [...subject.props().items, item],
      });
      expect(subject.find('.content').length).toBe(i + 1);
    });
  });

  it('should render two blocks block if prop isMobile is true', () => {
    const subject = makeSubject({
      isMobile: true,
    });

    expect(subject.find('.block').length).toBe(2);
  });

  it('should render the props regulationMsg on the empty block except to the first two block are prerequisiteMsg', () => {
    const subject = makeSubject({
      items: [],
    });

    Array.from(subject.find('.block .empty')).forEach((s, i) => {
      if (i < 2) {
        expect(
          subject
            .find('.block .empty')
            .at(i)
            .text()
        ).toBe(defaultProps.prerequisiteMsg);
      } else {
        expect(
          subject
            .find('.block .empty')
            .at(i)
            .text()
        ).toBe(defaultProps.regulationMsg);
      }
    });
  });

  it('should call prop handleClickClose if clicking the cross icon', () => {
    const subject = makeSubject();

    defaultProps.items.forEach((item, i) => {
      subject
        .find('.close')
        .at(i)
        .simulate('click');
      expect(defaultProps.clickItemCloseCb).toHaveBeenCalledWith(defaultProps.items[i].id);
    });
  });

  it('should call prop ctaCb if clicking the compare button', () => {
    const subject = makeSubject();
    const ctaBtn = subject.find('.cta');

    ctaBtn.simulate('click');

    expect(defaultProps.ctaCb).toHaveBeenCalledWith(defaultProps.items);
  });

  it('should contain GA event if given the gaEventForClose prop', () => {
    const gaEventForClose = {
      dataPrefix: ['data-proj-ga'],
      category: '基金比較清單',
      action: 'Click',
      label: '刪除基金',
    };
    const subject = makeSubject({ gaEventForClose });

    expect(
      subject
        .find('.close')
        .at(0)
        .prop('data-proj-ga-category')
    ).toBe('基金比較清單');
    expect(
      subject
        .find('.close')
        .at(0)
        .prop('data-proj-ga-action')
    ).toBe('Click');
    expect(
      subject
        .find('.close')
        .at(0)
        .prop('data-proj-ga-label')
    ).toBe('刪除基金');
  });

  it('should contain GA event if given the gaEventForStart prop', () => {
    const gaEventForStart = {
      dataPrefix: ['data-proj-ga'],
      category: '基金比較清單',
      action: 'Click',
      label: '開始比較',
    };
    const subject = makeSubject({ gaEventForStart });

    expect(subject.find('.cta').prop('data-proj-ga-category')).toBe('基金比較清單');
    expect(subject.find('.cta').prop('data-proj-ga-action')).toBe('Click');
    expect(subject.find('.cta').prop('data-proj-ga-label')).toBe('開始比較');
  });
});
